var __defProp = Object.defineProperty;
var __defProps = Object.defineProperties;
var __getOwnPropDescs = Object.getOwnPropertyDescriptors;
var __getOwnPropSymbols = Object.getOwnPropertySymbols;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __propIsEnum = Object.prototype.propertyIsEnumerable;
var __defNormalProp = (obj, key, value) => key in obj ? __defProp(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __spreadValues = (a, b) => {
  for (var prop in b || (b = {}))
    if (__hasOwnProp.call(b, prop))
      __defNormalProp(a, prop, b[prop]);
  if (__getOwnPropSymbols)
    for (var prop of __getOwnPropSymbols(b)) {
      if (__propIsEnum.call(b, prop))
        __defNormalProp(a, prop, b[prop]);
    }
  return a;
};
var __spreadProps = (a, b) => __defProps(a, __getOwnPropDescs(b));
var __accessCheck = (obj, member, msg) => {
  if (!member.has(obj))
    throw TypeError("Cannot " + msg);
};
var __privateGet = (obj, member, getter) => {
  __accessCheck(obj, member, "read from private field");
  return getter ? getter.call(obj) : member.get(obj);
};
var __privateAdd = (obj, member, value) => {
  if (member.has(obj))
    throw TypeError("Cannot add the same private member more than once");
  member instanceof WeakSet ? member.add(obj) : member.set(obj, value);
};
var __privateSet = (obj, member, value, setter) => {
  __accessCheck(obj, member, "write to private field");
  setter ? setter.call(obj, value) : member.set(obj, value);
  return value;
};
var __privateMethod = (obj, member, method) => {
  __accessCheck(obj, member, "access private method");
  return method;
};
var _manifest, _manifestData, _rootFolder, _routeDataToRouteInfo, _routeCache, _renderersPromise, _loadRenderers, loadRenderers_fn, _loadModule, loadModule_fn;
import { defaultLogOptions } from "../logger.js";
import { matchRoute } from "../routing/match.js";
import { render } from "../render/core.js";
import { RouteCache } from "../render/route-cache.js";
import { createLinkStylesheetElementSet, createModuleScriptElementWithSrcSet } from "../render/ssr-element.js";
import { prependForwardSlash } from "../path.js";
class App {
  constructor(manifest, rootFolder) {
    __privateAdd(this, _loadRenderers);
    __privateAdd(this, _loadModule);
    __privateAdd(this, _manifest, void 0);
    __privateAdd(this, _manifestData, void 0);
    __privateAdd(this, _rootFolder, void 0);
    __privateAdd(this, _routeDataToRouteInfo, void 0);
    __privateAdd(this, _routeCache, void 0);
    __privateAdd(this, _renderersPromise, void 0);
    __privateSet(this, _manifest, manifest);
    __privateSet(this, _manifestData, {
      routes: manifest.routes.map((route) => route.routeData)
    });
    __privateSet(this, _rootFolder, rootFolder);
    __privateSet(this, _routeDataToRouteInfo, new Map(manifest.routes.map((route) => [route.routeData, route])));
    __privateSet(this, _routeCache, new RouteCache(defaultLogOptions));
    __privateSet(this, _renderersPromise, __privateMethod(this, _loadRenderers, loadRenderers_fn).call(this));
  }
  match(request) {
    const url = new URL(request.url);
    return matchRoute(url.pathname, __privateGet(this, _manifestData));
  }
  async render(request, routeData) {
    if (!routeData) {
      routeData = this.match(request);
      if (!routeData) {
        return new Response(null, {
          status: 404,
          statusText: "Not found"
        });
      }
    }
    const manifest = __privateGet(this, _manifest);
    const info = __privateGet(this, _routeDataToRouteInfo).get(routeData);
    const [mod, renderers] = await Promise.all([__privateMethod(this, _loadModule, loadModule_fn).call(this, info.file), __privateGet(this, _renderersPromise)]);
    const url = new URL(request.url);
    const links = createLinkStylesheetElementSet(info.links, manifest.site);
    const scripts = createModuleScriptElementWithSrcSet(info.scripts, manifest.site);
    const result = await render({
      legacyBuild: false,
      links,
      logging: defaultLogOptions,
      markdownRender: manifest.markdown.render,
      mod,
      origin: url.origin,
      pathname: url.pathname,
      scripts,
      renderers,
      async resolve(specifier) {
        if (!(specifier in manifest.entryModules)) {
          throw new Error(`Unable to resolve [${specifier}]`);
        }
        const bundlePath = manifest.entryModules[specifier];
        return prependForwardSlash(bundlePath);
      },
      route: routeData,
      routeCache: __privateGet(this, _routeCache),
      site: __privateGet(this, _manifest).site,
      ssr: true,
      method: info.routeData.type === "endpoint" ? "" : "GET",
      headers: request.headers
    });
    if (result.type === "response") {
      return result.response;
    }
    let html = result.html;
    return new Response(html, {
      status: 200
    });
  }
}
_manifest = new WeakMap();
_manifestData = new WeakMap();
_rootFolder = new WeakMap();
_routeDataToRouteInfo = new WeakMap();
_routeCache = new WeakMap();
_renderersPromise = new WeakMap();
_loadRenderers = new WeakSet();
loadRenderers_fn = async function() {
  return await Promise.all(__privateGet(this, _manifest).renderers.map(async (renderer) => {
    const mod = await import(renderer.serverEntrypoint);
    return __spreadProps(__spreadValues({}, renderer), { ssr: mod.default });
  }));
};
_loadModule = new WeakSet();
loadModule_fn = async function(rootRelativePath) {
  let modUrl = new URL(rootRelativePath, __privateGet(this, _rootFolder)).toString();
  let mod;
  try {
    mod = await import(modUrl);
    return mod;
  } catch (err) {
    throw new Error(`Unable to import ${modUrl}. Does this file exist?`);
  }
};
export {
  App
};
