import { performance } from "perf_hooks";
import * as vite from "vite";
import { runHookConfigDone, runHookConfigSetup, runHookServerDone, runHookServerSetup, runHookServerStart } from "../../integrations/index.js";
import { createVite } from "../create-vite.js";
import { defaultLogOptions, info, warn } from "../logger.js";
import * as msg from "../messages.js";
import { apply as applyPolyfill } from "../polyfill.js";
import { getResolvedHostForVite } from "./util.js";
async function dev(config, options = { logging: defaultLogOptions }) {
  var _a;
  const devStart = performance.now();
  applyPolyfill();
  config = await runHookConfigSetup({ config, command: "dev" });
  const viteConfig = await createVite({
    mode: "development",
    server: { host: getResolvedHostForVite(config) }
  }, { astroConfig: config, logging: options.logging, mode: "dev" });
  await runHookConfigDone({ config });
  const viteServer = await vite.createServer(viteConfig);
  runHookServerSetup({ config, server: viteServer });
  await viteServer.listen(config.devOptions.port);
  const devServerAddressInfo = viteServer.httpServer.address();
  const site = config.buildOptions.site ? new URL(config.buildOptions.site) : void 0;
  info(options.logging, null, msg.devStart({ startupTime: performance.now() - devStart, config, devServerAddressInfo, site, https: !!((_a = viteConfig.server) == null ? void 0 : _a.https) }));
  const currentVersion = "0.25.0-next.2";
  if (currentVersion.includes("-")) {
    warn(options.logging, null, msg.prerelease({ currentVersion }));
  }
  await runHookServerStart({ config, address: devServerAddressInfo });
  return {
    address: devServerAddressInfo,
    stop: async () => {
      await viteServer.close();
      await runHookServerDone({ config });
    }
  };
}
export {
  dev as default
};
