import path from "path";
import { unwrapId, viteID } from "../../util.js";
const STYLE_EXTENSIONS = /* @__PURE__ */ new Set([".css", ".pcss", ".postcss", ".scss", ".sass", ".styl", ".stylus", ".less"]);
const cssRe = new RegExp(`\\.(${Array.from(STYLE_EXTENSIONS).map((s) => s.slice(1)).join("|")})($|\\?)`);
const isCSSRequest = (request) => cssRe.test(request);
function getStylesForURL(filePath, viteServer) {
  const importedCssUrls = /* @__PURE__ */ new Set();
  function crawlCSS(_id, isFile, scanned = /* @__PURE__ */ new Set()) {
    const id = unwrapId(_id);
    const importedModules = /* @__PURE__ */ new Set();
    const moduleEntriesForId = isFile ? viteServer.moduleGraph.getModulesByFile(id) ?? /* @__PURE__ */ new Set() : /* @__PURE__ */ new Set([viteServer.moduleGraph.getModuleById(id)]);
    for (const entry of moduleEntriesForId) {
      if (id === entry.id) {
        scanned.add(id);
        for (const importedModule of entry.importedModules) {
          importedModules.add(importedModule);
        }
      }
    }
    for (const importedModule of importedModules) {
      if (!importedModule.id || scanned.has(importedModule.id)) {
        continue;
      }
      const ext = path.extname(importedModule.url).toLowerCase();
      if (STYLE_EXTENSIONS.has(ext)) {
        importedCssUrls.add(importedModule.url);
      }
      crawlCSS(importedModule.id, false, scanned);
    }
  }
  crawlCSS(viteID(filePath), true);
  return importedCssUrls;
}
export {
  STYLE_EXTENSIONS,
  getStylesForURL,
  isCSSRequest
};
