var __defProp = Object.defineProperty;
var __defProps = Object.defineProperties;
var __getOwnPropDescs = Object.getOwnPropertyDescriptors;
var __getOwnPropSymbols = Object.getOwnPropertySymbols;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __propIsEnum = Object.prototype.propertyIsEnumerable;
var __defNormalProp = (obj, key, value) => key in obj ? __defProp(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __spreadValues = (a, b) => {
  for (var prop in b || (b = {}))
    if (__hasOwnProp.call(b, prop))
      __defNormalProp(a, prop, b[prop]);
  if (__getOwnPropSymbols)
    for (var prop of __getOwnPropSymbols(b)) {
      if (__propIsEnum.call(b, prop))
        __defNormalProp(a, prop, b[prop]);
    }
  return a;
};
var __spreadProps = (a, b) => __defProps(a, __getOwnPropDescs(b));
import { fileURLToPath } from "url";
import { render as coreRender } from "../core.js";
import { prependForwardSlash } from "../../../core/path.js";
import { createModuleScriptElementWithSrcSet } from "../ssr-element.js";
import { getStylesForURL } from "./css.js";
import { errorHandler } from "./error.js";
import { getHmrScript } from "./hmr.js";
import { injectTags } from "./html.js";
const svelteStylesRE = /svelte\?svelte&type=style/;
const rendererCache = /* @__PURE__ */ new Map();
async function loadRenderer(viteServer, renderer) {
  const { url } = await viteServer.moduleGraph.ensureEntryFromUrl(renderer.serverEntrypoint);
  const cachedRenderer = rendererCache.get(url);
  if (cachedRenderer) {
    return __spreadProps(__spreadValues({}, renderer), { ssr: cachedRenderer });
  }
  const mod = await viteServer.ssrLoadModule(url);
  rendererCache.set(url, mod.default);
  return __spreadProps(__spreadValues({}, renderer), { ssr: mod.default });
}
async function loadRenderers(viteServer, astroConfig) {
  return Promise.all(astroConfig._ctx.renderers.map((r) => loadRenderer(viteServer, r)));
}
async function preload({ astroConfig, filePath, viteServer }) {
  const renderers = await loadRenderers(viteServer, astroConfig);
  const mod = await viteServer.ssrLoadModule(fileURLToPath(filePath));
  return [renderers, mod];
}
async function render(renderers, mod, ssrOpts) {
  const { astroConfig, filePath, logging, mode, origin, pathname, method, headers, route, routeCache, viteServer } = ssrOpts;
  const legacy = astroConfig.buildOptions.legacyBuild;
  const scripts = createModuleScriptElementWithSrcSet(!legacy && mod.hasOwnProperty("$$metadata") ? Array.from(mod.$$metadata.hoistedScriptPaths()) : []);
  if (mod.hasOwnProperty("$$metadata") && mode === "development" && !legacy) {
    scripts.add({
      props: { type: "module", src: "/@vite/client" },
      children: ""
    });
    scripts.add({
      props: { type: "module", src: new URL("../../../runtime/client/hmr.js", import.meta.url).pathname },
      children: ""
    });
  }
  for (const script of astroConfig._ctx.scripts) {
    if (script.stage === "head-inline") {
      scripts.add({
        props: {},
        children: script.content
      });
    }
  }
  let links = /* @__PURE__ */ new Set();
  if (!legacy) {
    [...getStylesForURL(filePath, viteServer)].forEach((href) => {
      if (mode === "development" && svelteStylesRE.test(href)) {
        scripts.add({
          props: { type: "module", src: href },
          children: ""
        });
      } else {
        links.add({
          props: {
            rel: "stylesheet",
            href,
            "data-astro-injected": true
          },
          children: ""
        });
      }
    });
  }
  let content = await coreRender({
    legacyBuild: astroConfig.buildOptions.legacyBuild,
    links,
    logging,
    markdownRender: astroConfig.markdownOptions.render,
    mod,
    origin,
    pathname,
    scripts,
    async resolve(s) {
      if (!astroConfig.buildOptions.legacyBuild) {
        const [resolvedUrl, resolvedPath] = await viteServer.moduleGraph.resolveUrl(s);
        if (resolvedPath.includes("node_modules/.vite")) {
          return resolvedPath.replace(/.*?node_modules\/\.vite/, "/node_modules/.vite");
        }
        if (!resolvedUrl.startsWith(".") && !resolvedUrl.startsWith("/")) {
          return "/@id" + prependForwardSlash(resolvedUrl);
        }
        return "/@fs" + prependForwardSlash(resolvedPath);
      } else {
        return s;
      }
    },
    renderers,
    route,
    routeCache,
    site: astroConfig.buildOptions.site,
    ssr: astroConfig.buildOptions.experimentalSsr,
    method,
    headers
  });
  if ((route == null ? void 0 : route.type) === "endpoint" || content.type === "response") {
    return content;
  }
  const tags = [];
  if (mode === "development" && legacy) {
    tags.push({
      tag: "script",
      attrs: { type: "module" },
      children: await getHmrScript(),
      injectTo: "head"
    });
  }
  if (legacy) {
    [...getStylesForURL(filePath, viteServer)].forEach((href) => {
      if (mode === "development" && svelteStylesRE.test(href)) {
        tags.push({
          tag: "script",
          attrs: { type: "module", src: href },
          injectTo: "head"
        });
      } else {
        tags.push({
          tag: "link",
          attrs: {
            rel: "stylesheet",
            href,
            "data-astro-injected": true
          },
          injectTo: "head"
        });
      }
    });
  }
  let html = injectTags(content.html, tags);
  if (mode === "development" && astroConfig.buildOptions.legacyBuild) {
    const relativeURL = filePath.href.replace(astroConfig.projectRoot.href, "/");
    html = await viteServer.transformIndexHtml(relativeURL, html, pathname);
  }
  if (!/<!doctype html/i.test(html)) {
    html = "<!DOCTYPE html>\n" + content;
  }
  return {
    type: "html",
    html
  };
}
async function ssr(preloadedComponent, ssrOpts) {
  try {
    const [renderers, mod] = preloadedComponent;
    return await render(renderers, mod, ssrOpts);
  } catch (e) {
    await errorHandler(e, { viteServer: ssrOpts.viteServer, filePath: ssrOpts.filePath });
    throw e;
  }
}
export {
  loadRenderers,
  preload,
  render,
  ssr
};
