import { invalidateCompilation, isCached } from "./compile.js";
import { info } from "../core/logger.js";
import * as msg from "../core/messages.js";
async function trackCSSDependencies(opts) {
  const { viteDevServer, filename, deps, id } = opts;
  if (viteDevServer) {
    const mod = viteDevServer.moduleGraph.getModuleById(id);
    if (mod) {
      const cssDeps = (await Promise.all(Array.from(deps).map((spec) => {
        return this.resolve(spec, id);
      }))).filter(Boolean).map((dep) => dep.id);
      const { moduleGraph } = viteDevServer;
      const depModules = new Set(mod.importedModules);
      for (const dep of cssDeps) {
        depModules.add(moduleGraph.createFileOnlyEntry(dep));
      }
      moduleGraph.updateModuleInfo(mod, depModules, /* @__PURE__ */ new Set(), true);
      for (const dep of cssDeps) {
        this.addWatchFile(dep);
      }
    }
  }
}
async function handleHotUpdate(ctx, config, logging) {
  var _a;
  invalidateCompilation(config, ctx.file);
  const filtered = new Set(ctx.modules);
  const files = /* @__PURE__ */ new Set();
  for (const mod2 of ctx.modules) {
    if ((_a = mod2.id) == null ? void 0 : _a.endsWith(".astro?html-proxy&index=0.js")) {
      filtered.delete(mod2);
      continue;
    }
    if (mod2.file && isCached(config, mod2.file)) {
      filtered.add(mod2);
      files.add(mod2.file);
    }
    for (const imp of mod2.importers) {
      if (imp.file && isCached(config, imp.file)) {
        filtered.add(imp);
        files.add(imp.file);
      }
    }
  }
  for (const file2 of files) {
    invalidateCompilation(config, file2);
  }
  const mod = ctx.modules.find((m) => m.file === ctx.file);
  const file = ctx.file.replace(config.projectRoot.pathname, "/");
  if (ctx.file.endsWith(".astro")) {
    ctx.server.ws.send({ type: "custom", event: "astro:update", data: { file } });
  }
  if (mod == null ? void 0 : mod.isSelfAccepting) {
    info(logging, "astro", msg.hmr({ file }));
  } else {
    info(logging, "astro", msg.reload({ file }));
  }
  return Array.from(filtered);
}
export {
  handleHotUpdate,
  trackCSSDependencies
};
